﻿using System;
using System.Text;
using System.Windows;
using System.Windows.Media.Imaging;
using System.Threading;
using System.Net;
using System.IO;

namespace HttpVideo
{
	public partial class MainWindow
	{

		private Thread _mainThread;
		private HttpWebRequest _webRequest;

		private delegate void OnFrameReceivedCallback(byte[] frameBuffer);

		public MainWindow()
		{
			InitializeComponent();
		}

		private static string MD5Hash(string str)
		{
			if (string.IsNullOrEmpty(str))
			{
				return "";
			}
			var hashStrings = new StringBuilder();
			foreach (var hashDyte in new System.Security.Cryptography.MD5CryptoServiceProvider().ComputeHash(Encoding.UTF8.GetBytes(str)))
			{
				hashStrings.Append(hashDyte.ToString("X2"));
			}
			return hashStrings.ToString();
		}

		private void ReceiveFramesWorkFunc()
		{
			Stream streamResponse = null;
			try
			{
				OnFrameReceivedCallback callBack = FrameReceived;

				var webResponse = (HttpWebResponse)_webRequest.GetResponse();

				streamResponse = webResponse.GetResponseStream();
				if (streamResponse == null)
				{
					MessageBox.Show("There is no server response! Check the server address, channel name, username and password.",
						"Error:", MessageBoxButton.OK, MessageBoxImage.Error);
					return;
				}
				streamResponse.ReadTimeout = 30000;

				const int maxBufferSize = 8 * 1024 * 1024;
				var readBuffer = new byte[maxBufferSize];

				var isJpegStartFound = -1;

				var actualData = 0;
				var bufferOffset = 0;

				while (true)
				{
					var readen = streamResponse.Read(readBuffer, actualData, maxBufferSize - actualData);

					if (readen == 0)
						return;

					actualData += readen;

					if (actualData == maxBufferSize)
					{
						actualData -= bufferOffset;

						Array.Copy(readBuffer, bufferOffset, readBuffer, 0, actualData);
						bufferOffset = 0;
					}

					if (isJpegStartFound == -1)
					{
						for (; bufferOffset + 2 < actualData; bufferOffset++)
						{
							if (readBuffer[bufferOffset] == '\r' && readBuffer[bufferOffset + 1] == '\n' &&
								readBuffer[bufferOffset + 2] == 0xFF && readBuffer[bufferOffset + 3] == 0xD8)
							{
								isJpegStartFound = bufferOffset;
								break;
							}
						}
					}
					else
					{
						for (; bufferOffset + 1 < actualData; bufferOffset++)
						{
							if (readBuffer[bufferOffset] == 0xFF && readBuffer[bufferOffset + 1] == 0xD9)
							{
								var frameSize = bufferOffset - isJpegStartFound + 2;

								var frame = new byte[frameSize];

								Array.Copy(readBuffer, isJpegStartFound, frame, 0, frameSize);

								Application.Current.Dispatcher?.BeginInvoke(callBack, frame);

								isJpegStartFound = -1;
								bufferOffset++;
								break;
							}
							if (readBuffer[bufferOffset] == 0xFF && readBuffer[bufferOffset + 1] == 0xD8)
							{
								isJpegStartFound = bufferOffset;
							}
						}
					}
				}
			}
			catch (WebException)
			{
				streamResponse?.Close();
				streamResponse?.Dispose();
			}
			catch (ThreadAbortException)
			{
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void FrameReceived(byte[] framebuffer)
		{
			try
			{
				var decoder = new JpegBitmapDecoder(new MemoryStream(framebuffer), BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.Default);
				BitmapSource bitmapSource = decoder.Frames[0];
				frameRender.Source = bitmapSource;
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void connectButton_Click(object sender, RoutedEventArgs e)
		{
			_webRequest?.Abort();

			if (_mainThread != null && _mainThread.IsAlive)
				_mainThread.Abort();

			try
			{
				var address = tbCameraAdress.Text;

				if (address.IndexOf("http://", StringComparison.InvariantCultureIgnoreCase) == -1)
					address = "http://" + address;

				var channelName = tbChannelName.Text;
				var login = tbUserName.Text;
				var password = MD5Hash(tbPassword.Text);
				address += $"/mobile?channel={channelName}&login={login}&password={password}";

				_webRequest = (HttpWebRequest)WebRequest.Create(address);
				_webRequest.Method = "GET";
				_webRequest.Timeout = 5000;
				_webRequest.ReadWriteTimeout = 3000;

				_mainThread = new Thread(ReceiveFramesWorkFunc) { IsBackground = true };
				_mainThread.Start();
			}
			catch (Exception we)
			{
				MessageBox.Show(we.Message);
			}
		}
	}
}
