﻿using System;
using System.Globalization;
using System.Windows;
using System.Windows.Media;
using Alarus;
using Alarus.GUI;
using Alarus.RealTimeFrameProviding;

namespace RtVisualizer
{
	public class RtVisualizer : PluginVisualiser
	{
		public override DrawingVisual NullableDrawingVisual => _drawingVisual;

		private IDrawingPanel _drawingPanel;
		private readonly DrawingVisual _drawingVisual = new DrawingVisual();

		public RtVisualizer(Guid pluginId)
			: base(pluginId)
		{
		}

		public override void Initialize(Guid channelId, IPluginClientToolSet pluginToolset, IPluginVisualizerSet visualiserSet)
		{
			base.Initialize(channelId, pluginToolset, visualiserSet);

			_drawingPanel = visualiserSet.DrawingPanel;

			_drawingPanel.AddVisual(_drawingVisual);
		}

		public override void Clear()
		{
			DrawingContext dc = _drawingVisual.RenderOpen();
			dc.Close();
		}

		/// <summary>
		/// Example. In the event of an alarm, it draws a rectangle filled with arbitrary color into cells 
		/// and displays an arbitrary integer value as a string
		/// </summary>
		/// <param name="chId"></param>
		/// <param name="chEv"></param>
		/// <param name="isAlarm"></param>
		public override void ProcessEvent(Guid chId, RawEvent chEv, bool isAlarm)
		{
			if (isAlarm)
			{
				DrawingContext drawingContext = _drawingVisual.RenderOpen();

				double curWidth = _drawingPanel.PanelWidth;
				double curHeight = _drawingPanel.PanelHeight;

				var sampleRect = new Rect(0.3, 0.3, 0.4, 0.4);

				Rect normalRect = new Rect(
					sampleRect.X * curWidth,
					sampleRect.Y * curHeight,
					sampleRect.Width * curWidth,
					sampleRect.Height * curHeight
					);

				drawingContext.DrawRectangle(
					new SolidColorBrush(ConvertGuidToColor(Guid.NewGuid())),
					new Pen(Brushes.Yellow, 2.0),
					normalRect
					);

				var formattedText = new FormattedText(
					ConvertGuidToMyStringId(Guid.NewGuid()),
					CultureInfo.GetCultureInfo("ru-RU"),
					FlowDirection.LeftToRight,
					new Typeface("Verdana"),
					20,
					Brushes.Red);

				drawingContext.DrawText(formattedText, new Point(normalRect.X + normalRect.Width / 2, normalRect.Y));

				drawingContext.Close();
			}
		}

		private static Color ConvertGuidToColor(Guid guid)
		{
			System.Drawing.Color cl = System.Drawing.Color.FromArgb(BitConverter.ToInt32(guid.ToByteArray(), 0));
			return Color.FromArgb(150/*cl.A*/, cl.R, cl.G, cl.B);
		}
		private static string ConvertGuidToMyStringId(Guid guid)
		{
			byte[] bArr = guid.ToByteArray();
			byte tempId = bArr[15];
			return tempId.ToString(CultureInfo.InvariantCulture);
		}
	}
}
